package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Items.lore;
import static me.unurled.sacredrealms.sr.utils.Logger.warn;

import com.google.gson.annotations.Expose;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.Item;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SRPlayer {

  @Expose private UUID uuid = null;
  @Expose private HashMap<Attribute, Double> attributes = new HashMap<>();
  private HashMap<Attribute, HashMap<ItemStack, Double>> itemAttributes = new HashMap<>();
  @Expose private List<PotionEffect> potionEffects;
  @Expose private Inventory inventory;

  public SRPlayer(@NotNull Player player) {
    this.uuid = player.getUniqueId();
    this.inventory = player.getInventory();
    this.itemAttributes = new HashMap<>();
    this.potionEffects = (List<PotionEffect>) player.getActivePotionEffects();
  }

  @NotNull
  public UUID getUuid() {
    return uuid;
  }

  @NotNull
  public Inventory getInventory() {
    return inventory;
  }

  public void setInventory(Inventory inventory) {
    this.inventory = inventory;
  }

  @NotNull
  public HashMap<Attribute, Double> getAttributes() {
    return attributes;
  }

  public double getAttribute(@NotNull Attribute attribute) {
    return getAttribute(attribute, attribute.getDefaultValue());
  }

  @NotNull
  public Double getAttribute(@NotNull Attribute attribute, @NotNull Double defaultValue) {
    return attributes.getOrDefault(attribute, defaultValue);
  }

  public void setAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    attributes.put(attribute, value);
  }

  public void removeAttribute(@NotNull Attribute attribute) {
    attributes.remove(attribute);
  }

  @Nullable
  public Object getHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return null;
    }
    Object o =
        p.getInventory()
            .getItemInMainHand()
            .getItemMeta()
            .getPersistentDataContainer()
            .get(attribute.getKey(), attribute.getType());
    if (o == null) {
      setHandItemAttribute(attribute, attribute.getDefaultValue());
      return attribute.getDefaultValue();
    }
    return o;
  }

  public void setHandItemAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    if (checkIfValueIsOutOfBoundsForAttribute(attribute, value, p)) return;
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer().set(attribute.getKey(), PersistentDataType.DOUBLE, value);
    i.setItemMeta(im);
    lore(i);
    p.getInventory().setItemInMainHand(i);
  }

  private boolean checkIfValueIsOutOfBoundsForAttribute(
      @NotNull Attribute attribute, @NotNull Double value, Player p) {
    if (!attribute.getType().getComplexType().isInstance(value)) {
      return true;
    }
    if (attribute.getType().equals(PersistentDataType.DOUBLE)) {
      double d = value;
      if (d < attribute.getMinValue() || d > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + d
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + d);
        return true;
      }
    }
    return false;
  }

  public void removeHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer().remove(attribute.getKey());
    i.setItemMeta(im);
  }

  public void addPotionEffect(@NotNull PotionEffect effect) {
    potionEffects.add(effect);
  }

  public void removePotionEffect(@NotNull PotionEffect effect) {
    potionEffects.remove(effect);
  }

  public void clearPotionEffects() {
    potionEffects.clear();
  }

  @NotNull
  public List<PotionEffect> getPotionEffects() {
    if (potionEffects == null) potionEffects = new ArrayList<>();
    return potionEffects;
  }

  public void setPotionEffects(@NotNull List<PotionEffect> effects) {
    potionEffects.clear();
    potionEffects.addAll(effects);
  }

  public void setItemAttributes(
      @NotNull Attribute attribute, @NotNull ItemStack item, @NotNull Double value) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).put(item, value);
    } else {
      HashMap<ItemStack, Double> map = new HashMap<>();
      map.put(item, value);
      itemAttributes.put(attribute, map);
    }
  }

  public void removeItemAttributes(@NotNull Attribute attribute, @NotNull ItemStack item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      itemAttributes.get(attribute).remove(item);
    }
  }

  public @Nullable Double getItemAttributes(@NotNull Attribute attribute, @NotNull ItemStack item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    if (itemAttributes.containsKey(attribute)) {
      return itemAttributes.get(attribute).get(item);
    }
    return null;
  }

  public @NotNull HashMap<ItemStack, Double> getItemAttributes(@NotNull Attribute attribute) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    return itemAttributes.getOrDefault(attribute, new HashMap<>());
  }

  public @NotNull HashMap<Attribute, Double> getItemAttributes(Item item) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    HashMap<Attribute, Double> map = new HashMap<>();
    itemAttributes.forEach((k, v) -> map.put(k, v.get(item)));
    return map;
  }

  public @NotNull Double getTotalItemAttribute(@NotNull Attribute attribute) {
    if (itemAttributes == null) itemAttributes = new HashMap<>();
    return itemAttributes.getOrDefault(attribute, new HashMap<>()).values().stream()
        .reduce(0.0, Double::sum);
  }
}
